const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Gdk = imports.gi.Gdk;

const ExtensionUtils = imports.misc.extensionUtils;

const GTK_VERSION = Gtk.get_major_version();

function init() {
}

function buildPrefsWidget() {
    let widget = new PrefsWidget();
    	if(GTK_VERSION == 3) {
    		widget.widget.show_all(); }
    		
    return widget.widget;
}

class PrefsWidget {
    constructor() {
        this._settings = ExtensionUtils.getSettings();
        this.BC_button = new Gtk.ColorButton();
        this._setButtonColor();

        this.widget = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
        });

        this.vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            hexpand: true,
        });
        
        if(GTK_VERSION == 3) {
        	this.vbox.add(this.selectButtonColor());
        	this.vbox.add(this.resetButtonColor());
        	this.vbox.add(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.add(this.adjustIconSize());
        	this.vbox.add(this.adjustPadding());
        	this.vbox.add(this.adjustBorderRadius());
        	this.vbox.add(this.resetButton());
        	this.vbox.add(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.add(this.tip());
        	this.vbox.add(this.command());
        	
        	this.widget.add(this.vbox);        
        }
        
        else if(GTK_VERSION == 4) {
        	this.vbox.append(this.selectButtonColor());
        	this.vbox.append(this.resetButtonColor());
        	this.vbox.append(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.append(this.adjustIconSize());
        	this.vbox.append(this.adjustPadding());
        	this.vbox.append(this.adjustBorderRadius());
        	this.vbox.append(this.resetButton());
        	this.vbox.append(new Gtk.Separator({ orientation: Gtk.Orientation.HORIZONTAL, margin_bottom: 10, margin_top: 10 }));
        	this.vbox.append(this.tip());
        	this.vbox.append(this.command());
        	
        	this.widget.append(this.vbox);
    	}
    }
    
_setButtonColor() {
        let rgba = new Gdk.RGBA();
        let hexString = this._settings.get_string('color');
        rgba.parse(hexString);
        this.BC_button.set_rgba(rgba);
    }
    
    
_cssHexString(css) {
        let rrggbb = '#';
        let start;
        for(let loop = 0; loop < 3; loop++) {
            let end = 0;
            let xx = '';
            for(let loop = 0; loop < 2; loop++) {
                while(true) {
                    let x = css.slice(end, end + 1);
                    if(x == '(' || x == ',' || x == ')')
                        break;
                    end = end + 1;
                }
                if(loop == 0) {
                    end = end + 1;
                    start = end;
                }
            }
            xx = parseInt(css.slice(start, end)).toString(16);
            if(xx.length == 1)
                xx = '0' + xx;
            rrggbb = rrggbb + xx;
            css = css.slice(end);
        }
        return rrggbb;
    }
    
_onPanelColorChanged() {
        let rgba = this.BC_button.get_rgba();
        let css = rgba.to_string();
        let hexString = this._cssHexString(css);
        this._settings.set_string('color', hexString);
    }    
    
    
selectButtonColor() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
	let BC_label = new Gtk.Label({ label: "Select Button Color", xalign: 0, hexpand: true });
	
	this.BC_button.connect('notify::rgba', ()=> this._onPanelColorChanged() );
	
	if(GTK_VERSION == 3) {
		hbox.add(BC_label);
		hbox.add(this.BC_button);
		}
		
	else if(GTK_VERSION == 4) {
		hbox.append(BC_label);
		hbox.append(this.BC_button);
		}

        return hbox;
    }
    
resetButtonColor() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.END  });
    	let resetButtonColor = new Gtk.Button();
        resetButtonColor.set_label("Reset to Extensions's Default Color By Cliking this Button");
        resetButtonColor.connect('clicked', ()=> {	
        	this._settings.set_string('color', '#eeeeec');
        	this._setButtonColor();
       		});
	
	if(GTK_VERSION == 3) {
		hbox.add(resetButtonColor); }
	else if(GTK_VERSION == 4) {
		hbox.append(resetButtonColor) }

        return hbox;
    } 
    
adjustIconSize() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
	let IS_label = new Gtk.Label({ label: "Adjust Icon Size", xalign: 0, hexpand: true });

	this.IS_button = new Gtk.SpinButton();
	this.IS_button.set_range(12, 20);
	this.IS_button.set_value(this._settings.get_int('icon-size'));
	this.IS_button.set_increments(2, 4);
	this.IS_button.connect('value-changed', (entry) => { this._settings.set_int('icon-size', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(IS_label);
		hbox.add(this.IS_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(IS_label);
		hbox.append(this.IS_button); }

        return hbox;
    }
    
adjustPadding() {
        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let PD_label = new Gtk.Label({ label: "Adjust Padding", xalign: 0, hexpand: true });

        this.PD_button = new Gtk.SpinButton();
	this.PD_button.set_range(0, 16);
	this.PD_button.set_value(this._settings.get_int('padding'));
	this.PD_button.set_increments(2, 4);
	this.PD_button.connect('value-changed', (entry) => { this._settings.set_int('padding', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(PD_label);
		hbox.add(this.PD_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(PD_label);
		hbox.append(this.PD_button); }

        return hbox;
    }  
    
adjustBorderRadius() {
	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
	let BR_label = new Gtk.Label({ label: "Adjust Border Raidus", xalign: 0, hexpand: true });
	this.BR_button = new Gtk.SpinButton();
	this.BR_button.set_range(0, 32);
	this.BR_button.set_value(this._settings.get_int('border-radius'));
	this.BR_button.set_increments(2, 4);
	this.BR_button.connect('value-changed', (entry) => { this._settings.set_int('border-radius', entry.get_value()); });
	
	if(GTK_VERSION == 3) {
		hbox.add(BR_label);
		hbox.add(this.BR_button); }
		
	else if(GTK_VERSION == 4) {
		hbox.append(BR_label);
		hbox.append(this.BR_button); }

        return hbox;
    }
    
resetButton() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER  });
    	let resetButton = new Gtk.Button();
        resetButton.set_label("Reset to Extensions's Default Size'es By Cliking this Button");
        resetButton.connect('clicked', ()=> {	
        	this._settings.set_int('icon-size', 16);
        	this._settings.set_int('padding', 8);
        	this._settings.set_int('border-radius', 16);
			this.IS_button.set_value(this._settings.get_int('icon-size'));
			this.PD_button.set_value(this._settings.get_int('padding'));
			this.BR_button.set_value(this._settings.get_int('border-radius'));
			
       		});
	
	if(GTK_VERSION == 3) {
		hbox.add(resetButton); }
	else if(GTK_VERSION == 4) {
		hbox.append(resetButton) }

        return hbox;
    }  
    
tip() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER  });
    	let tipLabel = new Gtk.Label({ label:
    	"you can tweak the BACKGROUND COLOR of these system menu buttons by edititng this extensions stylesheet.css file. Run below command to open the styles sheet file.", wrap: true });
    	
    	if(GTK_VERSION == 3) {
		hbox.add(tipLabel); }
	else if(GTK_VERSION == 4) {
		hbox.append(tipLabel); }
		
        return hbox;
    }     
    
command() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5, halign: Gtk.Align.CENTER  });
    	let command = new Gtk.Label({ label: "gedit $HOME/.local/share/gnome-shell/extensions/roundedSystemMenuButtons@pratap.fastmail.fm/stylesheet.css", selectable: true });
    	
    	if(GTK_VERSION == 3) {
		hbox.add(command); }
        else if(GTK_VERSION == 4) {
        	hbox.append(command) }
      	
      	return hbox;
    }    
}

